/* $Id: ResourceUtility.java,v 1.6 2005/04/07 12:18:52 buchmand Exp $
 *
 * OPEN SOURCE LICENSE
 * (The "MIT License")
 *
 * Copyright (c) 2001-2004 nanoTITAN, Inc.
 * Enhanced 2005 by David Buchmann, University of Fribourg
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to 
 * deal in the Software without restriction, including without limitation the 
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * Visit www.nanotitan.com for more information. 
 *
 */

package nT.util;

import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.jar.*;


/*****************************************************************************
   This utilily class provides methods to find resources.
   
   <p>
   You can specify a prefix and a suffix the files must match. You can either 
   search the classes folders in the classpath, the jars or both.<br>
   Suffix can be a file name (ending), but can also contain the directory 
   containing the file. Directories must always be separated by / , even under
   windows.<br>
   Prefix is understood relative to the jar root / classes directory and can 
   contain / as directory separator.</p>
   <p>You can also search for directories. This happens when the suffix ends with
   a slash '/'.</p>
   
   @author nanoTitan
   @author david.buchmann at unifr.ch
****************************************************************************/

public class ResourceUtility
{

/****************************************************************************
   Finds and returns URLs of all the system resources that end with a particular
   suffix.  System resources will be found in the optional extensions folder
   and off of the CLASSPATH.
<p>
   * @param suffix the suffix to look for
<p>
   * @return a Collection of resources that match the search criteria
   * @throws IOException If creation of jar file fails.
*/
//____________________________________________________________________________

   public static final synchronized ArrayList
   getSystemResourcesBySuffix(String suffix)
   throws IOException
//____________________________________________________________________________
{
    return getSystemResources("", suffix);
}
/****************************************************************************
   Finds and returns URLs of all the system resources that are under a 
   particular directory.  System resources will be found in the optional
   extensions folder and off of the CLASSPATH.
<p>
   * @param prefix the directory to look in
<p>
   * @return a Collection of resources that match the search criteria
   * @throws IOException If creation of jar file fails.
*/
//____________________________________________________________________________

   public static final synchronized ArrayList
   getSystemResourcesByPrefix(String prefix)
   throws IOException
//____________________________________________________________________________
{
    return getSystemResources(prefix, "");
}


/****************************************************************************
   Finds and returns URLs of all the system resources under a specified 
   directory that end with a particular suffix.
   System resources will be found in the optional extensions folder and off of
   the CLASSPATH.
   If prefix ends with /, all sub-directories will be searched as well. 
   Otherwise, the resources file names have to begin with the prefix part after 
   the last /.
   If suffix ends with /, you are looking for a directory. Note that a directory
   can not be loaded as resource, but you may want to know if and where a 
   directory exists.
<p>
   * @param prefix the prefix to look for
   * @param suffix the suffix to look for
<p>
   * @return a Collection of resources that match the search criteria
   * @throws IOException If creation of jar file fails. 
*/
//____________________________________________________________________________

   public static final synchronized ArrayList
   getSystemResources(String prefix, String suffix)
   throws IOException
//____________________________________________________________________________
{
   ArrayList resourceList = new ArrayList();


   String javaHome = System.getProperty("java.home");
   File extDir = new File(javaHome + File.separator + "lib/ext");
   File[] extJars = extDir.listFiles();

   //int numJars = extJars.length;
   File jar = null;
   String jarStr = null;
   for(int j=(extJars.length-1); j>-1; j--)
   {
      jar = extJars[j];

      // Don't check some known JARs
      jarStr = jar.getName();


      if(   (jarStr.endsWith(".jar")) &&
            (jarStr.equals("j3dcore.jar") == false) &&
            (jarStr.equals("j3daudio.jar") == false) &&
            (jarStr.equals("j3dutils.jar") == false) &&
            (jarStr.equals("vecmath.jar") == false) &&
            (jarStr.equals("sunrsasign.jar") == false) &&
            (jarStr.equals("jsdt.jar") == false) &&
            (jarStr.equals("xml4j.jar") == false) &&
            (jarStr.equals("jDataConnect.jar") == false) &&
            (jarStr.equals("j2ee.jar") == false) &&
            (jarStr.equals("pvxAll.jar") == false) &&
            (jarStr.equals("dt.jar") == false) &&
            (jarStr.equals("tools.jar") == false)         )
      {
	  resourceList.addAll(getResourcesFromArchive( new JarFile(jar),jar,prefix,suffix));
      }
   }



   // LOOK IN THE CLASSPATH
   String classPathStr = System.getProperty("java.class.path");
   StringTokenizer st = new StringTokenizer(classPathStr,File.pathSeparator);
   String path = null;
   File file = null;
   while(st.hasMoreTokens())
   {
      path = st.nextToken();
      file = new File(path);
      if(file.exists())
      {
         if(file.isDirectory())
         {
	     resourceList.addAll(getResourcesFromDirectory(file,prefix,suffix));
         }
         else if(path.endsWith(".jar"))
         {
	     resourceList.addAll(getResourcesFromArchive(new JarFile(file),file,prefix,suffix));
         }
      }
   }


   return resourceList;
}


/****************************************************************************
   Finds all of the entries within the specified JAR file that have the
   specified suffix and returns the associated resource URLs.
<p>
   * @param jar the JAR file to search; if null an empty list will be returned
   * @param file the actual File reference to the JAR file
   * @param suffix the suffix for which to search
<p>
   * @return a Collection of resources that match the search criteria
*/
//____________________________________________________________________________

   public static final ArrayList
   getResourcesFromArchiveBySuffix(JarFile jar, File file, String suffix)
//____________________________________________________________________________
{
    return getResourcesFromArchive(jar,file,"",suffix);
}

/****************************************************************************
   Finds all of the entries within the specified JAR file that are under a
   specified prefix and returns the associated resource URLs.
<p>
   * @param jar the JAR file to search; if null an empty list will be returned
   * @param file the actual File reference to the JAR file
   * @param prefix the directory to search in
<p>
   * @return a Collection of resources that match the search criteria
*/
//____________________________________________________________________________

   public static final ArrayList
       getResourcesFromArchiveByPrefix(JarFile jar, File file, String prefix)
//____________________________________________________________________________
{
    return getResourcesFromArchive(jar,file,prefix,"");
}

/****************************************************************************
   Finds all of the entries within the specified JAR file that have the
   specified suffix and returns the associated resource URLs.
<p>
   * @param jar the JAR file to search; if null an empty list will be returned
   * @param file the actual File reference to the JAR file
   * @param prefix the directory to search
   * @param suffix the suffix for which to search
<p>
   * @return a Collection of resources that match the search criteria
*/
//____________________________________________________________________________

   public static final ArrayList
       getResourcesFromArchive(JarFile jar, File file, String prefix, String suffix)
//____________________________________________________________________________
{
   ArrayList resourceList = new ArrayList();
   if(jar == null) return resourceList;
   if (prefix.startsWith("/")) prefix = prefix.substring(1); //remove leading /

   
   
   // Create the URL prefix
   String urlprefix = null;
   try
   {
      urlprefix = "jar:" + file.toURL().toString() + "!/";
   }
   catch(Exception e)
   {
      e.printStackTrace();
      return resourceList;
   }
   // Process each entry in JAR file
   Enumeration e = jar.entries();
   JarEntry entry = null;
   URL entryURL = null;
   String entryName = null;
   while(e.hasMoreElements())
   {
      entry = (JarEntry) e.nextElement();
      //if(entry.isDirectory() == false) 
      //if the suffix ends with /, we get directory, otherwise not
      //{
         entryName = entry.getName();
         if(entryName.startsWith(prefix) && entryName.endsWith(suffix)) {
         	try
			{
			entryURL = new URL(urlprefix + entryName);
			if (resourceList.contains(entryURL) == false) resourceList.add(entryURL);
		    }
		catch(Exception e2)
		    {
			e2.printStackTrace();
		    }
	     }

   }
   //}

   return resourceList;
}


/****************************************************************************
   Finds all of the files from entry point dir, which are contained within the
   subdirectories prefix and returns the associated resource URLs.
<p>
   * @param dir the directory to add (potentially); if dir is null or dir
                is not a directory then an empty Collection is returned
   * @param prefix the prefix for which to search
<p>
   * @return a Collection of URLs of resources that match the specified prefix
*/
//____________________________________________________________________________

   public static final ArrayList
   getResourcesFromDirectoryByPrefix(File dir, String prefix)
//____________________________________________________________________________
{
    return getResourcesFromDirectory(dir, prefix, "");
}
/****************************************************************************
   Finds all of the files within the specified directory and subdirectories
   that have the specified suffix and returns the associated resource URLs.
<p>
   * @param dir the directory to add (potentially); if dir is null or dir
                is not a directory then an empty Collection is returned
   * @param suffix the suffix for which to search
<p>
   * @return a Collection of URLs of resources that match the specified suffix
*/
//____________________________________________________________________________

   public static final ArrayList
   getResourcesFromDirectoryBySuffix(File dir, String suffix)
//____________________________________________________________________________
{
    return getResourcesFromDirectory(dir, "", suffix);
}

/****************************************************************************
   Finds all of the files under the specified root directory within 
   subdirectories specified by prefix that have the specified suffix and 
   returns the associated resource URLs.
<p>
   * @param dir the directory to add (potentially); if dir is null or dir
                is not a directory then an empty Collection is returned
   * @param prefix the prefix for which to search
   * @param suffix the suffix for which to search
<p>
   * @return a Collection of URLs of resources that match the specified criteria
*/
//____________________________________________________________________________

   public static final ArrayList
       getResourcesFromDirectory(File dir, String prefix, String suffix)
//____________________________________________________________________________
{
	if (prefix.startsWith("/")) prefix=prefix.substring(1);
  	return getResourcesFromDirectory(dir, prefix, suffix, "");
}

   /** Implements the recursive search through directories.
    * 
    * @param dir The root directory of the search
    * @param prefix Prefix to match against path under root directory
    * @param suffix Suffix to match against the files / directories
    * @param relpath Path so far opened from root path.
    * @return A Collection of URLs of resources that match the specified criteria
    */
   private static final ArrayList
   getResourcesFromDirectory(File dir, String prefix, String suffix, String relpath) {
   	
   ArrayList resourceList = new ArrayList();
   if(dir == null) return resourceList;
   File current = new File(dir, relpath);
   if(! current.isDirectory()) return resourceList;

   // Process each file in directory
   String[] files = current.list();
   File f = null;
   URL url = null;
   for(int i=(files.length-1); i>-1; i--)
   {
      f = new File(current, files[i]);
      if(f.exists())
      {
      	if(f.isDirectory())
      		{
      		String newpath = relpath+files[i]+"/";		
      		if (newpath.startsWith(prefix) && newpath.endsWith(suffix)) 
      			resourceList.add(newpath);
      		
      		resourceList.addAll(getResourcesFromDirectory(dir,prefix,suffix,newpath)); //prefix is in f now
      		}
      	else if(f.isFile())
         {
            if((relpath+files[i]).startsWith(prefix) && (relpath+files[i]).endsWith(suffix))
            {
               try
               {
                  url = f.toURL();
                  if(resourceList.contains(url) == false) resourceList.add(url);
               }
               catch(Exception exc)
               {
                  exc.printStackTrace();
               }
            }
         }
      }
   }


   return resourceList;
}
}
